@@ -175,16 +175,12 @@ static void on_fullscreen_toggled(GtkToggleAction *, Chmsee *);
 static void on_sidepane_toggled(GtkToggleAction *, Chmsee *);
 
 static void populate_windows(Chmsee *);
-static void chmsee_set_fullscreen(Chmsee *, gboolean);
-static void show_sidepane(Chmsee *);
-static void hide_sidepane(Chmsee *);
+static void set_fullscreen(Chmsee *, gboolean);
 static void set_sidepane_state(Chmsee *, gboolean);
-static void chmsee_open_draged_file(Chmsee *, const gchar *);
-static void chmsee_drag_data_received(GtkWidget *, GdkDragContext *, gint, gint,
-                                      GtkSelectionData *, guint, guint);
+static void open_draged_file(Chmsee *, const gchar *);
+static void drag_data_received(GtkWidget *, GdkDragContext *, gint, gint,
+                               GtkSelectionData *, guint, guint);
 static void update_status_bar(Chmsee *, const gchar *);
-static void fullscreen(Chmsee *);
-static void unfullscreen(Chmsee *);
 
 /* Normal items */
 static const GtkActionEntry entries[] = {
@@ -210,8 +206,8 @@ static const GtkActionEntry entries[] = {
         { "Home", GTK_STOCK_HOME, N_("_Home"), NULL, NULL, G_CALLBACK(on_home)},
         { "Back", GTK_STOCK_GO_BACK, N_("_Back"), "<alt>Left", NULL, G_CALLBACK(on_back)},
         { "Forward", GTK_STOCK_GO_FORWARD, N_("_Forward"), "<alt>Right", NULL, G_CALLBACK(on_forward)},
-        { "Prev", GTK_STOCK_GO_UP, N_("_Prev"), "<alt>Up", NULL, G_CALLBACK(on_prev)},
-        { "Next", GTK_STOCK_GO_DOWN, N_("_Next"), "<alt>Down", NULL, G_CALLBACK(on_next)},
+        { "Prev", GTK_STOCK_GO_UP, N_("_Prev"), "<control>Up", NULL, G_CALLBACK(on_prev)},
+        { "Next", GTK_STOCK_GO_DOWN, N_("_Next"), "<control>Down", NULL, G_CALLBACK(on_next)},
 
         { "About", GTK_STOCK_ABOUT, N_("_About"), NULL, N_("About ChmSee"), G_CALLBACK(on_about)},
 
@@ -225,7 +221,7 @@ static const GtkActionEntry entries[] = {
 
 /* Toggle items */
 static const GtkToggleActionEntry toggle_entries[] = {
-        { "FullScreen", NULL, N_("Full _Screen"), "F11", "Switch between full screen and windowed mode", G_CALLBACK(on_fullscreen_toggled), FALSE },
+        { "FullScreen", NULL, N_("Full _Screen"), "F11", "Switch between fullscreen and window mode", G_CALLBACK(on_fullscreen_toggled), FALSE },
         { "SidePane", NULL, N_("Side _Pane"), "F9", NULL, G_CALLBACK(on_sidepane_toggled), FALSE }
 };
 
@@ -233,6 +229,12 @@ static const GtkToggleActionEntry toggle_entries[] = {
 static const GtkRadioActionEntry radio_entries[] = {
 };
 
+static const gchar *active_actions[] = {
+        "NewTab", "CloseTab", "SelectAll", "Home", "Find", "SidePane",
+        "ZoomIn", "ZoomOut", "ZoomReset", "Back", "Forward", "Prev", "Next",
+        ""
+};
+
 /* GObject functions */
 
 G_DEFINE_TYPE (Chmsee, chmsee, GTK_TYPE_WINDOW);
@@ -247,7 +249,7 @@ chmsee_class_init(ChmseeClass *klass)
         object_class->finalize = chmsee_finalize;
         object_class->dispose  = chmsee_dispose;
 
-        GTK_WIDGET_CLASS(klass)->drag_data_received = chmsee_drag_data_received;
+        GTK_WIDGET_CLASS(klass)->drag_data_received = drag_data_received;
 }
 
 static void
@@ -257,10 +259,10 @@ chmsee_init(Chmsee *self)
 
         priv->chmfile = NULL;
         priv->config  = NULL;
-        priv->state = CHMSEE_STATE_INIT;
+        priv->state   = CHMSEE_STATE_INIT;
 
         gtk_widget_add_events(GTK_WIDGET(self),
-                              GDK_STRUCTURE_MASK | GDK_BUTTON_PRESS_MASK );
+                              GDK_STRUCTURE_MASK | GDK_BUTTON_PRESS_MASK);
 
         g_signal_connect(G_OBJECT (self),
                          "window-state-event",
@@ -279,7 +281,7 @@ chmsee_init(Chmsee *self)
                          G_CALLBACK (delete_cb),
                          self);
 
-        /* startup html render engine */
+        /* start up html render engine */
         if(!cs_html_gecko_init_system()) {
                 g_error("Initialize html render engine failed!");
                 exit(1);
@@ -292,16 +294,15 @@ static void
 chmsee_dispose(GObject *gobject)
 {
         g_debug("Chmsee >>> dispose");
-        Chmsee        *self = CHMSEE(gobject);
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
+        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (CHMSEE (gobject));
 
-        if (priv->chmfile) {
+        if (priv->chmfile != NULL) {
                 /* save last opened page */
                 const gchar *bookfolder = cs_chmfile_get_bookfolder(priv->chmfile);
                 gchar *location = cs_book_get_location(CS_BOOK (priv->book));
                 gchar *page = g_strrstr(location, bookfolder);
 
-                if (page) {
+                if (page != NULL) {
                         page = page + strlen(bookfolder);
                         gchar *last_file = g_strdup_printf("%s::%s", priv->config->last_file, page);
                         g_free(priv->config->last_file);
@@ -314,7 +315,7 @@ chmsee_dispose(GObject *gobject)
                 priv->chmfile = NULL;
         }
 
-        if (priv->action_group) {
+        if (priv->action_group != NULL) {
                 g_object_unref(priv->action_group);
                 g_object_unref(priv->ui_manager);
                 priv->action_group = NULL;
@@ -345,7 +346,7 @@ delete_cb(GtkWidget *widget, GdkEvent *event, Chmsee *self)
 static gboolean
 window_state_event_cb(Chmsee *self, GdkEventWindowState *event)
 {
-        g_return_val_if_fail(IS_CHMSEE(self), FALSE);
+        g_return_val_if_fail(IS_CHMSEE (self), FALSE);
         g_return_val_if_fail(event->type == GDK_WINDOW_STATE, FALSE);
 
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
@@ -355,25 +356,29 @@ window_state_event_cb(Chmsee *self, GdkEventWindowState *event)
                 event->new_window_state
                 );
 
-        if(!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)) {
-                return FALSE;
-        }
-
-        if(event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
-                if(priv->config->fullscreen) {
-                        fullscreen(self);
+        if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
+                if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
+                        if (priv->config->fullscreen) {
+                                priv->config->fullscreen = TRUE;
+                                gtk_widget_hide(priv->menubar);
+                                gtk_widget_hide(priv->toolbar);
+                                gtk_widget_hide(priv->statusbar);
+                        } else {
+                                g_warning("expect not fullscreen but got a fullscreen event, restored");
+                                set_fullscreen(self, FALSE);
+                                return TRUE;
+                        }
                 } else {
-                        g_warning("expect not fullscreen but got a fullscreen event, restored");
-                        chmsee_set_fullscreen(self, FALSE);
-                        return TRUE;
-                }
-        } else {
-                if(!priv->config->fullscreen) {
-                        unfullscreen(self);
-                } else {
-                        g_warning("expect fullscreen but got an unfullscreen event, restored");
-                        chmsee_set_fullscreen(self, TRUE);
-                        return TRUE;
+                        if (!priv->config->fullscreen) {
+                                priv->config->fullscreen = FALSE;
+                                gtk_widget_show(priv->menubar);
+                                gtk_widget_show(priv->toolbar);
+                                gtk_widget_show(priv->statusbar);
+                        } else {
+                                g_warning("expect fullscreen but got an unfullscreen event, restored");
+                                set_fullscreen(self, TRUE);
+                                return TRUE;
+                        }
                 }
         }
 
@@ -402,26 +407,17 @@ book_model_changed_cb(Chmsee *self, CsChmfile *chmfile, const gchar *filename)
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
         gboolean has_model = (chmfile != NULL);
 
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "NewTab"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "CloseTab"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "SelectAll"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Home"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Find"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "SidePane"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "ZoomIn"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "ZoomOut"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "ZoomReset"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Back"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Forward"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Prev"), has_model);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Next"), has_model);
+        gint n = 0;
+        while (strlen(active_actions[n])) {
+                gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, active_actions[n++]), has_model);
+        }
 
         gtk_widget_set_sensitive(priv->book, has_model);
 
         if (filename
             && g_str_has_prefix(filename, "file://")
             && (g_str_has_suffix(filename, ".chm") || g_str_has_suffix(filename, ".CHM"))) {
-                chmsee_open_draged_file(self, filename);
+                open_draged_file(self, filename);
         }
 }
 
@@ -430,16 +426,14 @@ book_html_changed_cb(Chmsee *self, CsBook *book)
 {
         g_debug("Chmsee >>> recieve html_changed signal");
 
-        gboolean home_state, back_state, forward_state;
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
 
-        home_state = cs_book_has_homepage(book);
-        back_state = cs_book_can_go_back(book);
-        forward_state = cs_book_can_go_forward(book);
-
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Home"), home_state);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Back"), back_state);
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Forward"), forward_state);
+        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Home"),
+                                 cs_book_has_homepage(book));
+        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Back"),
+                                 cs_book_can_go_back(book));
+        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Forward"),
+                                 cs_book_can_go_forward(book));
 }
 
 static void
@@ -462,10 +456,10 @@ open_file_response_cb(GtkWidget *widget, gint response_id, Chmsee *self)
 
         gtk_widget_destroy(widget);
 
-        if (filename != NULL)
+        if (filename != NULL) {
                 chmsee_open_file(self, filename);
-
-        g_free(filename);
+                g_free(filename);
+        }
 }
 
 /* Toolbar button events */
@@ -516,28 +510,25 @@ on_recent_files(GtkRecentChooser *chooser, Chmsee *self)
 {
         gchar *uri = gtk_recent_chooser_get_current_uri(chooser);
 
-        if (uri != NULL)
-        {
+        if (uri != NULL) {
                 gchar *filename = g_filename_from_uri(uri, NULL, NULL);
 
                 chmsee_open_file(self, filename);
                 g_free(filename);
+                g_free(uri);
         }
-        g_free(uri);
 }
 
 static void
 on_copy(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_copy(CS_BOOK (priv->book));
+        cs_book_copy(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_select_all(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_select_all(CS_BOOK (priv->book));
+        cs_book_select_all(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
@@ -549,52 +540,49 @@ on_setup(GtkAction *action, Chmsee *self)
 static void
 on_back(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_go_back(CS_BOOK (priv->book));
+        cs_book_go_back(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_forward(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_go_forward(CS_BOOK (priv->book));
+        cs_book_go_forward(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_prev(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_go_prev(CS_BOOK (priv->book));
+        cs_book_go_prev(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_next(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_go_next(CS_BOOK (priv->book));
+        cs_book_go_next(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_menu_file(GtkAction *action, Chmsee *self)
 {
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        gboolean can_close_tab = cs_book_can_close_tab(CS_BOOK (priv->book));
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "CloseTab"), can_close_tab);
+
+        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "CloseTab"),
+                                 cs_book_can_close_tab(CS_BOOK (priv->book)));
 }
 
 static void
 on_menu_edit(GtkAction *action, Chmsee *self)
 {
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        gboolean can_copy = cs_book_can_copy(CS_BOOK (priv->book));
-        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Copy"), can_copy);
+
+        gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, "Copy"),
+                                 cs_book_can_copy(CS_BOOK (priv->book)));
 }
 
 static void
 on_home(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_homepage(CS_BOOK (priv->book));
+        cs_book_homepage(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
@@ -625,7 +613,6 @@ about_response_cb(GtkDialog *dialog, gint response_id, gpointer user_data)
 static void
 on_about(GtkAction *action)
 {
-
         GtkBuilder *builder = gtk_builder_new();
         gtk_builder_add_from_file(builder, RESOURCE_FILE ("about-dialog.ui"), NULL);
 
@@ -637,6 +624,8 @@ on_about(GtkAction *action)
                          NULL);
 
         gtk_about_dialog_set_version(GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
+        gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG (dialog),
+                                      _("(Build with xulrunner " XULRUNNER_VERSION ")"));
 
         g_object_unref(builder);
 }
@@ -660,38 +649,25 @@ on_close_current_tab(GtkAction *action, Chmsee *self)
 static void
 on_keyboard_escape(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-
-        cs_book_findbar_hide(CS_BOOK (priv->book));
+        cs_book_findbar_hide(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 static void
 on_fullscreen_toggled(GtkToggleAction *action, Chmsee *self)
 {
-        g_return_if_fail(IS_CHMSEE(self));
-
-        gboolean active = gtk_toggle_action_get_active(action);
-        chmsee_set_fullscreen(self, active);
+        set_fullscreen(self, gtk_toggle_action_get_active(action));
 }
 
 static void
 on_sidepane_toggled(GtkToggleAction *action, Chmsee *self)
 {
-        g_return_if_fail(IS_CHMSEE (self));
-
-        gboolean active = gtk_toggle_action_get_active(action);
-        if (active) {
-                show_sidepane(self);
-        } else {
-                hide_sidepane(self);
-        }
+        set_sidepane_state(self, gtk_toggle_action_get_active(action));
 }
 
 static void
 on_find(GtkAction *action, Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_book_findbar_show(CS_BOOK (priv->book));
+        cs_book_findbar_show(CS_BOOK (CHMSEE_GET_PRIVATE (self)->book));
 }
 
 /* internal functions */
@@ -711,23 +687,23 @@ on_quit(GtkAction *action, Chmsee *self)
 }
 
 static void
-chmsee_drag_data_received(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
-                          GtkSelectionData *selection_data, guint info, guint time)
+drag_data_received(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+                   GtkSelectionData *selection_data, guint info, guint time)
 {
-        gchar  **uris;
-        gint     i = 0;
+        gchar **uris;
 
         uris = gtk_selection_data_get_uris(selection_data);
-        if (!uris) {
-                gtk_drag_finish (context, FALSE, FALSE, time);
+        if (uris == NULL) {
+                gtk_drag_finish(context, FALSE, FALSE, time);
                 return;
         }
 
+        gint i;
         for (i = 0; uris[i]; i++) {
                 gchar *uri = uris[i];
                 if (g_str_has_prefix(uri, "file://")
                     && (g_str_has_suffix(uri, ".chm") || g_str_has_suffix(uri, ".CHM"))) {
-                        chmsee_open_draged_file(CHMSEE (widget), uri);
+                        open_draged_file(CHMSEE (widget), uri);
                         break;
                 }
         }
@@ -738,9 +714,9 @@ chmsee_drag_data_received(GtkWidget *widget, GdkDragContext *context, gint x, gi
 }
 
 static void
-chmsee_open_draged_file(Chmsee *chmsee, const gchar *file)
+open_draged_file(Chmsee *chmsee, const gchar *file)
 {
-        gchar *fname = g_uri_unescape_string(file+7, NULL);
+        gchar *fname = g_uri_unescape_string(file+7, NULL); // +7 remove "file://" prefix
         chmsee_open_file(chmsee, fname);
         g_free(fname);
 }
@@ -752,49 +728,34 @@ populate_windows(Chmsee *self)
 
         GtkWidget *vbox = GTK_WIDGET (gtk_vbox_new(FALSE, 2));
 
-        GtkActionGroup *action_group = gtk_action_group_new("MenuActions");
-        priv->action_group = action_group;
+        priv->action_group = gtk_action_group_new("MenuActions");
         gtk_action_group_set_translation_domain(priv->action_group, NULL);
-        gtk_action_group_add_actions(action_group, entries, G_N_ELEMENTS (entries), self);
-        gtk_action_group_add_toggle_actions(action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), self);
+        gtk_action_group_add_actions(priv->action_group, entries, G_N_ELEMENTS (entries), self);
+        gtk_action_group_add_toggle_actions(priv->action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), self);
 
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "NewTab"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "CloseTab"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "SelectAll"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Find"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Home"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Back"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Forward"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Prev"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "Next"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "SidePane"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "ZoomIn"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "ZoomOut"), FALSE);
-        gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "ZoomReset"), FALSE);
-
-        GtkUIManager *ui_manager = gtk_ui_manager_new();
-        priv->ui_manager = ui_manager;
-        gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
-
-        GtkAccelGroup *accel_group = gtk_ui_manager_get_accel_group(ui_manager);
-        gtk_window_add_accel_group(GTK_WINDOW (self), accel_group);
-
-        GError *error = NULL;
-        if (!gtk_ui_manager_add_ui_from_string(ui_manager, ui_description, -1, &error))
-        {
-                g_warning ("Chmsee >>> building menus failed: %s", error->message);
-                g_error_free (error);
-                exit (EXIT_FAILURE);
+        gint n = 0;
+        while (strlen(active_actions[n])) {
+                gtk_action_set_sensitive(gtk_action_group_get_action(priv->action_group, active_actions[n++]), FALSE);
         }
 
-        GtkWidget *menubar = gtk_handle_box_new();
-        priv->menubar = menubar;
-        gtk_container_add(GTK_CONTAINER(menubar), gtk_ui_manager_get_widget (ui_manager, "/MainMenu"));
-        gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+        priv->ui_manager = gtk_ui_manager_new();
+        gtk_ui_manager_insert_action_group(priv->ui_manager, priv->action_group, 0);
 
-        GtkRecentManager *manager = gtk_recent_manager_get_default();
-        GtkWidget *recent_menu = gtk_recent_chooser_menu_new_for_manager(manager);
+        gtk_window_add_accel_group(GTK_WINDOW (self),
+                                   gtk_ui_manager_get_accel_group(priv->ui_manager));
 
+        GError *error = NULL;
+        if (!gtk_ui_manager_add_ui_from_string(priv->ui_manager, ui_description, -1, &error)) {
+                g_warning("Chmsee >>> building menus failed: %s", error->message);
+                g_error_free(error);
+                exit(EXIT_FAILURE);
+        }
+
+        priv->menubar = gtk_handle_box_new();
+        gtk_container_add(GTK_CONTAINER (priv->menubar), gtk_ui_manager_get_widget(priv->ui_manager, "/MainMenu"));
+        gtk_box_pack_start(GTK_BOX (vbox), priv->menubar, FALSE, FALSE, 0);
+
+        GtkWidget *recent_menu = gtk_recent_chooser_menu_new_for_manager(gtk_recent_manager_get_default());
         gtk_recent_chooser_set_show_not_found(GTK_RECENT_CHOOSER (recent_menu), FALSE);
         gtk_recent_chooser_set_local_only(GTK_RECENT_CHOOSER (recent_menu), TRUE);
         gtk_recent_chooser_set_limit(GTK_RECENT_CHOOSER (recent_menu), 10);
@@ -811,17 +772,16 @@ populate_windows(Chmsee *self)
                          G_CALLBACK (on_recent_files),
                          self);
 
-        GtkWidget *widget = gtk_ui_manager_get_widget(ui_manager, "/MainMenu/FileMenu/RecentFiles");
-        gtk_menu_item_set_submenu(GTK_MENU_ITEM (widget), recent_menu);
+        gtk_menu_item_set_submenu(GTK_MENU_ITEM (gtk_ui_manager_get_widget(priv->ui_manager, "/MainMenu/FileMenu/RecentFiles")),
+                                  recent_menu);
 
-        GtkWidget *toolbar = gtk_handle_box_new();
-        priv->toolbar = toolbar;
-        gtk_container_add(GTK_CONTAINER(toolbar), gtk_ui_manager_get_widget(ui_manager, "/toolbar"));
-        gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
+        priv->toolbar = gtk_handle_box_new();
+        gtk_container_add(GTK_CONTAINER (priv->toolbar), gtk_ui_manager_get_widget(priv->ui_manager, "/toolbar"));
+        gtk_box_pack_start(GTK_BOX (vbox), priv->toolbar, FALSE, FALSE, 0);
         /* gtk_toolbar_set_style(GTK_TOOLBAR (gtk_ui_manager_get_widget(ui_manager, "/toolbar")), */
         /*                       GTK_TOOLBAR_ICONS);// FIXME: issue 43 */
         gtk_tool_button_set_icon_widget(
-                GTK_TOOL_BUTTON(gtk_ui_manager_get_widget(ui_manager, "/toolbar/sidepane")),
+                GTK_TOOL_BUTTON(gtk_ui_manager_get_widget(priv->ui_manager, "/toolbar/sidepane")),
                 gtk_image_new_from_file(RESOURCE_FILE ("show-pane.png")));
 
         priv->book = cs_book_new();
@@ -839,9 +799,6 @@ populate_windows(Chmsee *self)
 
         gtk_container_add(GTK_CONTAINER (self), vbox);
 
-        accel_group = g_object_new(GTK_TYPE_ACCEL_GROUP, NULL);
-        gtk_window_add_accel_group(GTK_WINDOW (self), accel_group);
-
         update_status_bar(self, _("Ready!"));
         gtk_widget_show_all(GTK_WIDGET (self));
         cs_book_findbar_hide(CS_BOOK (priv->book));
@@ -849,19 +806,19 @@ populate_windows(Chmsee *self)
 }
 
 static void
-chmsee_set_fullscreen(Chmsee *self, gboolean fullscreen)
+set_fullscreen(Chmsee *self, gboolean fullscreen)
 {
-        g_debug("Chmsee >>> chmsee_set_fullscreen with fullscreen = %d", fullscreen);
+        g_debug("Chmsee >>> set_fullscreen with fullscreen = %d", fullscreen);
 
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
         priv->config->fullscreen = fullscreen;
 
-        if(fullscreen) {
+        if (fullscreen) {
                 g_debug("Chmsee >>> call gtk_window_fullscreen");
-                gtk_window_fullscreen(GTK_WINDOW(self));
+                gtk_window_fullscreen(GTK_WINDOW (self));
         } else {
                 g_debug("ChmSee >>> call gtk_window_unfullscreen");
-                gtk_window_unfullscreen(GTK_WINDOW(self));
+                gtk_window_unfullscreen(GTK_WINDOW (self));
         }
 }
 
@@ -882,58 +839,19 @@ set_sidepane_state(Chmsee *self, gboolean state)
 {
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
 
-        GtkWidget *icon_widget;
         g_object_set(priv->book,
                      "sidepane-visible", state,
                      NULL);
+        const gchar *pane_icon = state ? RESOURCE_FILE ("hide-pane.png") : RESOURCE_FILE ("show-pane.png");
 
-        if (state) {
-                icon_widget = gtk_image_new_from_file(RESOURCE_FILE ("hide-pane.png"));
-        } else {
-                icon_widget = gtk_image_new_from_file(RESOURCE_FILE ("show-pane.png"));
-        }
+        GtkWidget *icon_widget = gtk_image_new_from_file(pane_icon);
         gtk_widget_show(icon_widget);
+
         gtk_tool_button_set_icon_widget(
-                GTK_TOOL_BUTTON(gtk_ui_manager_get_widget(priv->ui_manager, "/toolbar/sidepane")),
+                GTK_TOOL_BUTTON (gtk_ui_manager_get_widget(priv->ui_manager, "/toolbar/sidepane")),
                 icon_widget);
 };
 
-static void
-show_sidepane(Chmsee *self)
-{
-        set_sidepane_state(self, TRUE);
-}
-
-static void
-hide_sidepane(Chmsee *self)
-{
-        set_sidepane_state(self, FALSE);
-}
-
-static void
-fullscreen(Chmsee *self)
-{
-        g_debug("Chmsee >>> enter fullscreen");
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-
-        priv->config->fullscreen = TRUE;
-        gtk_widget_hide(priv->menubar);
-        gtk_widget_hide(priv->toolbar);
-        gtk_widget_hide(priv->statusbar);
-}
-
-static void
-unfullscreen(Chmsee *self)
-{
-        g_message("Chmsee >>> enter unfullscreen");
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-
-        priv->config->fullscreen = FALSE;
-        gtk_widget_show(priv->menubar);
-        gtk_widget_show(priv->toolbar);
-        gtk_widget_show(priv->statusbar);
-}
-
 /* External functions */
 
 Chmsee *
@@ -947,16 +865,17 @@ chmsee_new(CsConfig *config)
         if (config->pos_x >= 0 && config->pos_y >= 0)
                 gtk_window_move(GTK_WINDOW (self), config->pos_x, config->pos_y);
 
-        if (config->width > 0 && config->height > 0)
-                gtk_window_resize(GTK_WINDOW (self), config->width, config->height);
-        else
-                gtk_window_resize(GTK_WINDOW (self), 800, 600);
+        if (config->width <= 0 || config->height <= 0) {
+                config->width = DEFAULT_WIDTH;
+                config->height = DEFAULT_HEIGHT;
+        }
+        gtk_window_resize(GTK_WINDOW (self), config->width, config->height);
 
         gtk_window_set_title(GTK_WINDOW (self), "ChmSee");
         gtk_window_set_icon_from_file(GTK_WINDOW (self), RESOURCE_FILE ("chmsee-icon.png"), NULL);
 
         cs_book_set_hpaned_position(CS_BOOK (priv->book), config->hpaned_pos);
-        hide_sidepane(self);
+        set_sidepane_state(self, FALSE); //hide
 
         /* widget size changed event handle */
         g_signal_connect(G_OBJECT (self),
@@ -976,15 +895,24 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
         g_message("Chmsee >>> open file = %s", filename);
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
 
-        /* close currently opened book */
-        if (priv->chmfile) {
-                g_object_unref(priv->chmfile);
-        }
-
         /* create chmfile, get file infomation */
-        priv->chmfile = cs_chmfile_new(filename, priv->config->bookshelf);
+        CsChmfile *new_chmfile = cs_chmfile_new(filename, priv->config->bookshelf);
+
+        if (new_chmfile != NULL) {
+                /* close currently opened book */
+                if (priv->chmfile != NULL)
+                        g_object_unref(priv->chmfile);
+
+                priv->chmfile = new_chmfile;
+
+                /* set global charset and font to this file */
+                if (!strlen(cs_chmfile_get_charset(priv->chmfile)) && strlen(priv->config->charset))
+                        cs_chmfile_set_charset(priv->chmfile, priv->config->charset);
+                if (!strlen(cs_chmfile_get_variable_font(priv->chmfile)))
+                        cs_chmfile_set_variable_font(priv->chmfile, priv->config->variable_font);
+                if (!strlen(cs_chmfile_get_fixed_font(priv->chmfile)))
+                        cs_chmfile_set_fixed_font(priv->chmfile, priv->config->fixed_font);
 
-        if (priv->chmfile) {
                 priv->state = CHMSEE_STATE_LOADING;
 
                 cs_book_set_model(CS_BOOK (priv->book), priv->chmfile);
@@ -1008,7 +936,7 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
                 g_free(window_title);
 
                 /* record last opened file */
-                if (priv->config->last_file)
+                if (priv->config->last_file != NULL)
                         g_free(priv->config->last_file);
 
                 priv->config->last_file = g_strdup(cs_chmfile_get_filename(priv->chmfile));
@@ -1019,7 +947,7 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
                 gsize length;
 
                 if (g_file_get_contents(filename, &content, &length, NULL)) {
-                        static gchar *groups[2] = {
+                        static gchar *groups[] = {
                                 "CHM Viewer",
                                 NULL
                         };
@@ -1028,10 +956,11 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
                         data->display_name = NULL;
                         data->description = NULL;
                         data->mime_type = "application/x-chm";
-                        data->app_name = (gchar*) g_get_application_name();
-                        data->app_exec = g_strjoin(" ", g_get_prgname (), "%u", NULL);
+                        data->app_name = (gchar*)g_get_application_name();
+                        data->app_exec = g_strjoin(" ", g_get_prgname(), "%u", NULL);
                         data->groups = groups;
                         data->is_private = FALSE;
+
                         gchar *uri = g_filename_to_uri(filename, NULL, NULL);
 
                         GtkRecentManager *manager = gtk_recent_manager_get_default();
@@ -1044,6 +973,8 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
 
                 priv->state = CHMSEE_STATE_NORMAL;
         } else {
+                g_warning("CS_BOOK >>> Can not open spectified file %s", filename);
+
                 /* Popup an error message dialog */
                 GtkWidget *msg_dialog;
 
@@ -1053,6 +984,7 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
                                                     GTK_BUTTONS_CLOSE,
                                                     _("Error: Can not open spectified file '%s'"),
                                                     filename);
+                gtk_window_set_position(GTK_WINDOW (msg_dialog), GTK_WIN_POS_CENTER);
                 gtk_dialog_run(GTK_DIALOG (msg_dialog));
                 gtk_widget_destroy(msg_dialog);
         }
@@ -1061,100 +993,134 @@ chmsee_open_file(Chmsee *self, const gchar *filename)
 gboolean
 chmsee_get_startup_lastfile(Chmsee *self)
 {
+        g_return_val_if_fail(IS_CHMSEE (self), FALSE);
+
         return CHMSEE_GET_PRIVATE (self)->config->startup_lastfile;
 }
 
 void
 chmsee_set_startup_lastfile(Chmsee *self, gboolean state)
 {
+        g_return_if_fail(IS_CHMSEE (self));
+
         g_debug("Chmsee >>> set startup lastfile = %d", state);
         CHMSEE_GET_PRIVATE (self)->config->startup_lastfile = state;
 }
 
-gboolean
-chmsee_get_splash(Chmsee *self)
-{
-        return CHMSEE_GET_PRIVATE (self)->config->splash;
-}
-
-void
-chmsee_set_splash(Chmsee *self, gboolean state)
-{
-        g_debug("Chmsee >>> set splash = %d", state);
-        CHMSEE_GET_PRIVATE (self)->config->splash = state;
-}
-
 const gchar *
 chmsee_get_variable_font(Chmsee *self)
 {
+        g_return_val_if_fail(IS_CHMSEE (self), NULL);
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        return cs_chmfile_get_variable_font(priv->chmfile);
+        if (priv->chmfile != NULL)
+                return cs_chmfile_get_variable_font(priv->chmfile);
+        else
+                return priv->config->variable_font;
 }
 
 void
 chmsee_set_variable_font(Chmsee *self, const gchar *font_name)
 {
+        g_return_if_fail(IS_CHMSEE (self));
+
         g_debug("Chmsee >>> set variable font = %s", font_name);
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_html_gecko_set_variable_font(font_name);
-        cs_chmfile_set_variable_font(priv->chmfile, font_name);
+
+        if (priv->chmfile != NULL) {
+                cs_html_gecko_set_variable_font(font_name);
+                cs_chmfile_set_variable_font(priv->chmfile, font_name);
+        } else {
+                g_free(priv->config->variable_font);
+                priv->config->variable_font = g_strdup(font_name);
+        }
 }
 
 const gchar *
 chmsee_get_fixed_font(Chmsee *self)
 {
+        g_return_val_if_fail(IS_CHMSEE (self), NULL);
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        return cs_chmfile_get_fixed_font(priv->chmfile);
+
+        if (priv->chmfile != NULL)
+                return cs_chmfile_get_fixed_font(priv->chmfile);
+        else
+                return priv->config->fixed_font;
 }
 
 void
 chmsee_set_fixed_font(Chmsee *self, const gchar *font_name)
 {
+        g_return_if_fail(IS_CHMSEE (self));
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_html_gecko_set_fixed_font(font_name);
-        cs_chmfile_set_fixed_font(priv->chmfile, font_name);
+
+        if (priv->chmfile != NULL) {
+                cs_html_gecko_set_fixed_font(font_name);
+                cs_chmfile_set_fixed_font(priv->chmfile, font_name);
+        } else {
+                g_free(priv->config->fixed_font);
+                priv->config->fixed_font = g_strdup(font_name);
+        }
 }
 
 const gchar *
 chmsee_get_charset(Chmsee *self)
 {
+        g_return_val_if_fail(IS_CHMSEE (self), NULL);
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        return cs_chmfile_get_charset(priv->chmfile);
+        if (priv->chmfile != NULL)
+                return cs_chmfile_get_charset(priv->chmfile);
+        else
+                return priv->config->charset;
 }
 
 void
 chmsee_set_charset(Chmsee *self, const gchar *charset)
 {
+        g_return_if_fail(IS_CHMSEE (self));
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        cs_chmfile_set_charset(priv->chmfile, charset);
-        cs_book_reload_current_page(CS_BOOK (priv->book));
+
+        if (priv->chmfile != NULL) {
+                cs_chmfile_set_charset(priv->chmfile, charset);
+                cs_book_reload_current_page(CS_BOOK (priv->book));
+        } else {
+                g_free(priv->config->charset);
+                priv->config->charset = g_strdup(charset);
+        }
 }
 
 gboolean
 chmsee_has_book(Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        return priv->chmfile != NULL;
+        g_return_val_if_fail(IS_CHMSEE (self), FALSE);
+
+        return CHMSEE_GET_PRIVATE (self)->chmfile != NULL;
 }
 
 void
 chmsee_close_book(Chmsee *self)
 {
+        g_return_if_fail(IS_CHMSEE (self));
+
         ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
 
-        if (priv->chmfile) {
+        if (priv->chmfile != NULL) {
                 g_object_unref(priv->chmfile);
                 priv->chmfile = NULL;
         }
 
         book_model_changed_cb(self, NULL, NULL);
-
         priv->state = CHMSEE_STATE_NORMAL;
 }
 
 const gchar *
 chmsee_get_bookshelf(Chmsee *self)
 {
-        ChmseePrivate *priv = CHMSEE_GET_PRIVATE (self);
-        return priv->config->bookshelf;
+        g_return_val_if_fail(IS_CHMSEE (self), NULL);
+
+        return CHMSEE_GET_PRIVATE (self)->config->bookshelf;
 }
