@@ -60,6 +60,9 @@ struct _CsConfig {
         gchar   *home;
         gchar   *bookshelf;
         gchar   *last_file;
+        gchar   *charset;
+        gchar   *variable_font;
+        gchar   *fixed_font;
 
         gint     pos_x;
         gint     pos_y;
@@ -68,7 +71,6 @@ struct _CsConfig {
         gint     hpaned_pos;
         gboolean fullscreen;
         gboolean startup_lastfile;
-        gboolean splash;
 };
 
 GType        chmsee_get_type(void);
