@@ -95,7 +95,7 @@ cs_bookmarks_init(CsBookmarks *self)
 
         /* bookmarks list */
         GtkWidget *frame = gtk_frame_new(NULL);
-        gtk_frame_set_shadow_type(GTK_FRAME (frame), GTK_SHADOW_IN);
+        gtk_frame_set_shadow_type(GTK_FRAME (frame), GTK_SHADOW_NONE);
 
         GtkWidget *list_sw = gtk_scrolled_window_new(NULL, NULL);
         gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (list_sw),
@@ -145,7 +145,6 @@ cs_bookmarks_init(CsBookmarks *self)
         gtk_box_pack_start(GTK_BOX (self), hbox, FALSE, FALSE, 2);
 
         gtk_box_set_spacing(GTK_BOX (self), 2);
-        gtk_container_set_border_width(GTK_CONTAINER (self), 2);
 
         gtk_widget_show_all(GTK_WIDGET (self));
 }
@@ -154,8 +153,8 @@ static void
 cs_bookmarks_finalize(GObject *object)
 {
         g_debug("CS_BOOKMARKS >>> finalize");
-        CsBookmarks        *self = CS_BOOKMARKS (object);
-        CsBookmarksPrivate *priv = CS_BOOKMARKS_GET_PRIVATE (self);
+
+        CsBookmarksPrivate *priv = CS_BOOKMARKS_GET_PRIVATE (CS_BOOKMARKS (object));
 
         g_free(priv->current_uri);
         priv->links = NULL;
@@ -168,10 +167,10 @@ cs_bookmarks_finalize(GObject *object)
 static void
 link_selected_cb(CsBookmarks *self, Link *link)
 {
-        if (link) {
-                g_debug("CS_BOOKMARKS >>> Emiting link-selected signal");
+        g_debug("CS_BOOKMARKS >>> Emiting link-selected signal");
+
+        if (link)
                 g_signal_emit(self, signals[LINK_SELECTED], 0, link);
-        }
 }
 
 static void
@@ -180,12 +179,9 @@ entry_changed_cb(GtkEntry *entry, CsBookmarks *self)
         CsBookmarksPrivate *priv = CS_BOOKMARKS_GET_PRIVATE (self);
 
         const gchar *name = gtk_entry_get_text(entry);
-        gint length = strlen(name);
+        gboolean sensitive = strlen(name) > 2 ? TRUE : FALSE;
 
-        if (length >= 3)
-                gtk_widget_set_sensitive(priv->add_button, TRUE);
-        else
-                gtk_widget_set_sensitive(priv->add_button, FALSE);
+        gtk_widget_set_sensitive(priv->add_button, sensitive);
 }
 
 static void
@@ -194,7 +190,7 @@ on_bookmarks_add(CsBookmarks *self)
         CsBookmarksPrivate *priv = CS_BOOKMARKS_GET_PRIVATE (self);
         g_debug("CS_BOOKMARKS >>> add button clicked, current_uri = %p", priv->current_uri);
 
-        if (!priv->current_uri)
+        if (priv->current_uri == NULL)
                 return;
 
         const gchar *name = gtk_entry_get_text(GTK_ENTRY (priv->entry));
