@@ -29,8 +29,6 @@
 #include <sys/stat.h>
 #include <gcrypt.h>
 #include <chm_lib.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtk.h>
 
 #include "chmfile.h"
 #include "utils.h"
@@ -67,13 +65,6 @@ struct extract_context
         const char *hhk_file;
 };
 
-struct extract_data
-{
-        const gchar *filename;
-        const gchar *bookfolder;
-        int          rval;
-};
-
 #define CS_CHMFILE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CS_TYPE_CHMFILE, CsChmfilePrivate))
 
 #define UINT16ARRAY(x) ((unsigned char)(x)[0] | ((u_int16_t)(x)[1] << 8))
@@ -103,7 +94,6 @@ static gboolean tree_to_list_callback(GNode *, gpointer);
 static void free_list_data(gpointer, gpointer);
 static gchar *check_file_ncase(CsChmfile *, const gchar *);
 
-static void extract_file(struct extract_data *);
 static void parse_filename(CsChmfile *, const gchar *);
 
 /* GObject functions */
@@ -139,8 +129,8 @@ cs_chmfile_init(CsChmfile *self)
         priv->homepage       = NULL;
 
         priv->encoding       = g_strdup("UTF-8");
-        priv->variable_font  = g_strdup("Sans 12");
-        priv->fixed_font     = g_strdup("Monospace 12");
+        priv->variable_font  = g_strdup("");
+        priv->fixed_font     = g_strdup("");
         priv->charset        = g_strdup("");
 }
 
@@ -186,19 +176,6 @@ cs_chmfile_finalize(GObject *object)
 
 /* Internal functions */
 
-static void
-extract_file(struct extract_data *data)
-{
-        gboolean rval = extract_chm(data->filename, data->bookfolder);
-
-        gdk_threads_enter();
-        if (rval)
-                data->rval = 0;
-        else
-                data->rval = 1;
-        gdk_threads_leave();
-}
-
 static int
 dir_exists(const char *path)
 {
@@ -210,11 +187,6 @@ dir_exists(const char *path)
 static int
 rmkdir(char *path)
 {
-        /*
-         * strip off trailing components unless we can stat the directory, or we
-         * have run out of components
-         */
-
         char *i = rindex(path, '/');
 
         if (path[0] == '\0'  ||  dir_exists(path))
@@ -231,7 +203,7 @@ rmkdir(char *path)
 }
 
 /*
- * callback function for enumerate API
+ * Callback function for enumerate API
  */
 static int
 _extract_callback(struct chmFile *h, struct chmUnitInfo *ui, void *context)
@@ -283,7 +255,7 @@ _extract_callback(struct chmFile *h, struct chmUnitInfo *ui, void *context)
                 while (remain != 0) {
                         len = chm_retrieve_object(h, ui, (unsigned char *)buffer, offset, 32768);
                         if (len > 0) {
-                                if(fwrite(buffer, 1, (size_t)len, fout) != len) {
+                                if (fwrite(buffer, 1, (size_t)len, fout) != len) {
                                         g_debug("CS_CHMFILE: CHM_ENUMERATOR_FAILURE fwrite");
                                         g_free(fname);
                                         return CHM_ENUMERATOR_FAILURE;
@@ -347,7 +319,7 @@ MD5File(const char *filename, char *buf)
         gcry_md_hd_t hd;
         int f, i;
 
-        if(filename == NULL)
+        if (filename == NULL)
                 return NULL;
 
         gcry_md_open(&hd, GCRY_MD_MD5, 0);
@@ -368,7 +340,7 @@ MD5File(const char *filename, char *buf)
         if (buf == NULL)
                 buf = malloc(33);
         if (buf == NULL)
-                return (NULL);
+                return NULL;
 
         digest = (unsigned char*)gcry_md_read(hd, 0);
 
@@ -461,6 +433,8 @@ get_encoding_by_lcid(guint32 lcid)
                 return "ISO-8859-2";
         case 0x0415:
                 return "WINDOWS-1250";
+        case 0x0419:
+                return "WINDOWS-1251";
         case 0x0c01:
                 return "WINDOWS-1256";
         case 0x0401:
@@ -511,7 +485,6 @@ get_encoding_by_lcid(guint32 lcid)
         case 0x0402:
         case 0x043f:
         case 0x042f:
-        case 0x0419:
         case 0x0c1a:
         case 0x0444:
         case 0x0422:
@@ -553,7 +526,7 @@ chmfile_file_info(CsChmfile *self)
         if (priv->hhc != NULL) {
                 gchar *new_hhc = check_file_ncase(self, priv->hhc);
 
-                if (new_hhc) {
+                if (new_hhc != NULL) {
                         g_free(priv->hhc);
                         priv->hhc = new_hhc;
                 }
@@ -562,13 +535,13 @@ chmfile_file_info(CsChmfile *self)
         if (priv->hhk != NULL) {
                 gchar *new_hhk = check_file_ncase(self, priv->hhk);
 
-                if (new_hhk) {
+                if (new_hhk != NULL) {
                         g_free(priv->hhk);
                         priv->hhk = new_hhk;
                 }
         }
 
-        /* convert encoding to UTF-8 */
+        /* Convert encoding to UTF-8 */
         if (priv->encoding != NULL) {
                 if (priv->bookname) {
                         g_debug("CS_CHMFILE >>> priv->bookname = %s", priv->bookname);
@@ -656,6 +629,10 @@ chmfile_windows_info(struct chmFile *cfd, CsChmfile *self)
                 priv->hhk = g_strdup_printf("/%s", buffer + hhk);
         if (priv->homepage == NULL && homepage)
                 priv->homepage = g_strdup_printf("/%s", buffer + homepage);
+        if (g_strcmp0(priv->homepage, "/") == 0 && homepage) {
+                g_free(priv->homepage);
+                priv->homepage = g_strdup_printf("/%s", buffer + homepage);
+        }
 
         if (priv->bookname == NULL && bookname)
                 priv->bookname = g_strdup((char *)buffer + bookname);
@@ -688,7 +665,7 @@ chmfile_system_info(struct chmFile *cfd, CsChmfile *self)
         for(;;) {
                 // This condition won't hold if I process anything
                 // except NUL-terminated strings!
-                if(index > size - 1 - (long)sizeof(u_int16_t))
+                if (index > size - 1 - (long)sizeof(u_int16_t))
                         break;
 
                 cursor = buffer + index;
@@ -716,7 +693,7 @@ chmfile_system_info(struct chmFile *cfd, CsChmfile *self)
                         cursor = buffer + index;
 
                         priv->homepage = g_strdup_printf("/%s", buffer + index + 2);
-                        g_debug("CS_CHMFILE >>> homepage %s", priv->homepage);
+                        g_debug("CS_CHMFILE >>> SYSTEM homepage %s", priv->homepage);
 
                         break;
                 case 3:
@@ -724,7 +701,7 @@ chmfile_system_info(struct chmFile *cfd, CsChmfile *self)
                         cursor = buffer + index;
 
                         priv->bookname = g_strdup((char *)buffer + index + 2);
-                        g_debug("CS_CHMFILE >>> bookname %s", priv->bookname);
+                        g_debug("CS_CHMFILE >>> SYSTEM bookname %s", priv->bookname);
 
                         break;
                 case 4: // LCID stuff
@@ -734,7 +711,7 @@ chmfile_system_info(struct chmFile *cfd, CsChmfile *self)
                         lcid = UINT32ARRAY(buffer + index + 2);
                         g_debug("CS_CHMFILE >>> lcid %x", lcid);
 
-                        if(priv->encoding)
+                        if (priv->encoding)
                                 g_free(priv->encoding);
 
                         priv->encoding = g_strdup(get_encoding_by_lcid(lcid));
@@ -744,7 +721,7 @@ chmfile_system_info(struct chmFile *cfd, CsChmfile *self)
                         index += 2;
                         cursor = buffer + index;
 
-                        if(!priv->hhc) {
+                        if (!priv->hhc) {
                                 char *hhc, *hhk;
 
                                 hhc = g_strdup_printf("/%s.hhc", buffer + index + 2);
@@ -811,28 +788,39 @@ load_bookinfo(CsChmfile *self)
         CsChmfilePrivate *priv = CS_CHMFILE_GET_PRIVATE (self);
 
         gchar *bookinfo_file = g_build_filename(priv->bookfolder, CHMSEE_BOOKINFO_FILE, NULL);
-
         g_debug("CS_CHMFILE >>> read bookinfo file = %s", bookinfo_file);
-
         GKeyFile *keyfile = g_key_file_new();
-
         gboolean rv = g_key_file_load_from_file(keyfile, bookinfo_file, G_KEY_FILE_NONE, NULL);
 
-        if (!rv) {
+        if (!rv)
                 convert_old_config_file(bookinfo_file, "[Bookinfo]\n");
-        }
 
         rv = g_key_file_load_from_file(keyfile, bookinfo_file, G_KEY_FILE_NONE, NULL);
 
         if (rv) {
-                priv->hhc           = g_key_file_get_string(keyfile, "Bookinfo", "hhc", NULL);
-                priv->hhk           = g_key_file_get_string(keyfile, "Bookinfo", "hhk", NULL);
-                priv->homepage      = g_key_file_get_string(keyfile, "Bookinfo", "homepage", NULL);
-                priv->bookname      = g_key_file_get_string(keyfile, "Bookinfo", "bookname", NULL);
-                priv->encoding      = g_key_file_get_string(keyfile, "Bookinfo", "encoding", NULL);
-                priv->variable_font = g_key_file_get_string(keyfile, "Bookinfo", "variable_font", NULL);
-                priv->fixed_font    = g_key_file_get_string(keyfile, "Bookinfo", "fixed_font", NULL);
-                priv->charset       = g_key_file_get_string(keyfile, "Bookinfo", "charset", NULL);
+                priv->hhc      = g_key_file_get_string(keyfile, "Bookinfo", "hhc", NULL);
+                priv->hhk      = g_key_file_get_string(keyfile, "Bookinfo", "hhk", NULL);
+                priv->homepage = g_key_file_get_string(keyfile, "Bookinfo", "homepage", NULL);
+                priv->bookname = g_key_file_get_string(keyfile, "Bookinfo", "bookname", NULL);
+                priv->encoding = g_key_file_get_string(keyfile, "Bookinfo", "encoding", NULL);
+
+                gchar *vfont = g_key_file_get_string(keyfile, "Bookinfo", "variable_font", NULL);
+                if (vfont) {
+                        g_free(priv->variable_font);
+                        priv->variable_font = vfont;
+                }
+
+                gchar *ffont = g_key_file_get_string(keyfile, "Bookinfo", "fixed_font", NULL);
+                if (ffont) {
+                        g_free(priv->fixed_font);
+                        priv->fixed_font = ffont;
+                }
+
+                gchar *charset = g_key_file_get_string(keyfile, "Bookinfo", "charset", NULL);
+                if (charset) {
+                        g_free(priv->charset);
+                        priv->charset = charset;
+                }
         }
 
         g_key_file_free(keyfile);
@@ -924,7 +912,6 @@ parse_filename(CsChmfile *self, const gchar *filename)
         } else {
                 priv->chm = g_strdup(filename);
         }
-        g_debug("CS_CHMFILE >>> priv->chm = %s, priv->page = %s", priv->chm, priv->page);
 }
 
 /* External functions */
@@ -937,61 +924,24 @@ cs_chmfile_new(const gchar *filename, const gchar *bookshelf)
 
         parse_filename(self, filename);
 
-        /* use chm file's MD5 as folder name */
+        if (!g_str_has_suffix(priv->chm, ".CHM") && !g_str_has_suffix(priv->chm, ".chm"))
+                return NULL;
+
+        /* Use chmfile's MD5 as the folder name */
         gchar *md5 = MD5File(priv->chm, NULL);
-        if(!md5) {
-                g_warning("CS_CHMFILE >>> Oops!! cannot calculate chmfile's MD5 value.");
+        if (!md5) {
+                g_warning("CS_CHMFILE >>> Oops!! Cannot calculate chmfile's MD5!");
                 return NULL;
         }
 
         priv->bookfolder = g_build_filename(bookshelf, md5, NULL);
         g_debug("CS_CHMFILE >>> book folder = %s", priv->bookfolder);
 
-        /* if this chmfile already exists in bookshelf, load it's bookinfo */
+        /* If this chmfile already exists in the bookshelf, load it's bookinfo file */
         if (g_file_test(priv->bookfolder, G_FILE_TEST_IS_DIR)) {
                 load_bookinfo(self);
         } else {
-                GtkWidget *popup_window = gtk_window_new(GTK_WINDOW_POPUP);
-                GtkWidget *progress = gtk_progress_bar_new();
-
-                gtk_container_add(GTK_CONTAINER (popup_window), progress);
-                gtk_window_set_position(GTK_WINDOW (popup_window), GTK_WIN_POS_CENTER);
-                gtk_widget_set_size_request(popup_window, 300, 40);
-                gtk_widget_show_all(popup_window);
-
-                struct extract_data data;
-
-                data.filename = priv->chm;
-                data.bookfolder = priv->bookfolder;
-                data.rval = -1;
-
-                g_thread_create((GThreadFunc) (extract_file), &data, FALSE, NULL);
-
-                /* display a faked progress bar */
-                gdouble percent = 0.0;
-                while (data.rval < 0 && percent <= 100.0) {
-                        gchar *message = g_strdup_printf(_("Processing... %.0f%% complete"), percent);
-                        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR (progress), percent / 100.0);
-                        gtk_progress_bar_set_text(GTK_PROGRESS_BAR (progress), message);
-
-                        while (gtk_events_pending())
-                                gtk_main_iteration();
-
-                        g_usleep(100000);
-                        percent += 5.0;
-                        g_free(message);
-                }
-
-                gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR (progress), 1.0);
-                gtk_progress_bar_set_text(GTK_PROGRESS_BAR (progress), _("Processing done. Loading..."));
-
-                while (gtk_events_pending())
-                        gtk_main_iteration();
-                g_usleep(200000);
-
-                gtk_widget_destroy(GTK_WIDGET (popup_window));
-
-                if (data.rval == 1) {
+                if (!extract_chm(filename, priv->bookfolder)) {
                         g_warning("CS_CHMFILE >>> extract_chm failed: %s", priv->chm);
                         return NULL;
                 }
@@ -1004,9 +954,9 @@ cs_chmfile_new(const gchar *filename, const gchar *bookshelf)
         g_debug("CS_CHMFILE >>> priv->hhk = %s", priv->hhk);
         g_debug("CS_CHMFILE >>> priv->homepage = %s", priv->homepage);
         g_debug("CS_CHMFILE >>> priv->bookname = %s", priv->bookname);
-        g_debug("CS_CHMFILE >>> priv->endcoding = %s", priv->encoding);
+        g_debug("CS_CHMFILE >>> priv->encoding = %s", priv->encoding);
 
-        /* parse hhc file */
+        /* Parse hhc file */
         if (priv->hhc != NULL && g_ascii_strcasecmp(priv->hhc, "(null)") != 0) {
                 gchar *hhcfile = g_build_filename(priv->bookfolder, priv->hhc, NULL);
 
@@ -1015,7 +965,7 @@ cs_chmfile_new(const gchar *filename, const gchar *bookshelf)
                 g_free(hhcfile);
         }
 
-        /* parse hhk file */
+        /* Parse hhk file */
         if (priv->hhk != NULL && priv->index_list == NULL) {
                 gchar *hhkfile = g_build_filename(priv->bookfolder, priv->hhk, NULL);
 
@@ -1122,7 +1072,6 @@ cs_chmfile_set_variable_font(CsChmfile *self, const gchar *font_name)
         CsChmfilePrivate *priv = CS_CHMFILE_GET_PRIVATE (self);
 
         g_free(priv->variable_font);
-
         priv->variable_font = g_strdup(font_name);
 }
 
@@ -1139,7 +1088,6 @@ cs_chmfile_set_fixed_font(CsChmfile *self, const gchar *font_name)
         CsChmfilePrivate *priv = CS_CHMFILE_GET_PRIVATE (self);
 
         g_free(priv->fixed_font);
-
         priv->fixed_font = g_strdup(font_name);
 }
 
@@ -1155,6 +1103,5 @@ cs_chmfile_set_charset(CsChmfile *self, const gchar *charset)
         CsChmfilePrivate *priv = CS_CHMFILE_GET_PRIVATE (self);
 
         g_free(priv->charset);
-
         priv->charset = g_strdup(charset);
 }
