@@ -54,29 +54,26 @@ static gboolean find_charset_func(GtkTreeModel *, GtkTreePath *,
 static void
 on_bookshelf_clear(GtkWidget *widget, Chmsee *chmsee)
 {
-        char *argv[4];
-        gchar *bookshelf = g_strdup(chmsee_get_bookshelf(chmsee));
+        const gchar *bookshelf = chmsee_get_bookshelf(chmsee);
 
-        chmsee_close_book(chmsee);
+        if (bookshelf && g_file_test(bookshelf, G_FILE_TEST_EXISTS)) {
+                chmsee_close_book(chmsee);
 
-        g_return_if_fail(g_file_test(bookshelf, G_FILE_TEST_EXISTS));
+                gchar *dir = strdup(bookshelf);
+                char *argv[] = {"rm", "-rf", dir, NULL};
 
-        argv[0] = "rm";
-        argv[1] = "-rf";
-        argv[2] = bookshelf;
-        argv[3] = NULL;
-
-        g_spawn_async(g_get_tmp_dir(), argv, NULL,
-                      G_SPAWN_SEARCH_PATH,
-                      NULL, NULL, NULL,
-                      NULL);
-        g_free(bookshelf);
+                g_spawn_async(g_get_tmp_dir(), argv, NULL,
+                              G_SPAWN_SEARCH_PATH,
+                              NULL, NULL, NULL,
+                              NULL);
+                g_free(dir);
+        }
 }
 
 static void
 on_window_close(GtkButton *button, Chmsee *chmsee)
 {
-        gtk_widget_destroy(gtk_widget_get_toplevel (GTK_WIDGET(button)));
+        gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET (button)));
 }
 
 static void
@@ -112,15 +109,17 @@ cmb_lang_changed_cb(GtkComboBox *combo_box, Chmsee *chmsee)
                            -1);
 
         g_debug("SETUP >>> select charset: %s", charset);
-        chmsee_set_charset(chmsee, charset);
+        if (charset && strlen(charset))
+                chmsee_set_charset(chmsee, charset);
+        else
+                chmsee_set_charset(chmsee, "Auto");
 }
 
 static void
 startup_lastfile_toggled_cb(GtkWidget *widget, Chmsee *chmsee)
 {
         g_debug("SETUP >>> startup_lastfile toggled");
-        GtkToggleButton *toggle_button = GTK_TOGGLE_BUTTON (widget);
-        gboolean state = gtk_toggle_button_get_active(toggle_button);
+        gboolean state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widget));
 
         chmsee_set_startup_lastfile(chmsee, state);
 }
@@ -268,8 +267,7 @@ static void
 cell_layout_data_func(GtkCellLayout *layout, GtkCellRenderer *renderer,
                       GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
-        gboolean sensitive = !gtk_tree_model_iter_has_child(model, iter);
-        gtk_cell_renderer_set_sensitive(renderer, sensitive);
+        gtk_cell_renderer_set_sensitive(renderer, !gtk_tree_model_iter_has_child(model, iter));
 }
 
 static gboolean
@@ -340,31 +338,25 @@ setup_window_new(Chmsee *chmsee)
                                            cell_layout_data_func,
                                            NULL, NULL);
 
-        if (chmsee_has_book(chmsee)) {
-                gtk_font_button_set_font_name(GTK_FONT_BUTTON (variable_font_button),
-                                              chmsee_get_variable_font(chmsee));
-                gtk_font_button_set_font_name(GTK_FONT_BUTTON (fixed_font_button),
-                                              chmsee_get_fixed_font(chmsee));
-                gtk_widget_set_sensitive(variable_font_button, TRUE);
-                gtk_widget_set_sensitive(fixed_font_button, TRUE);
+        gtk_font_button_set_font_name(GTK_FONT_BUTTON (variable_font_button),
+                                      chmsee_get_variable_font(chmsee));
+        gtk_font_button_set_font_name(GTK_FONT_BUTTON (fixed_font_button),
+                                      chmsee_get_fixed_font(chmsee));
 
-                const gchar *charset = chmsee_get_charset(chmsee);
-                g_debug("SETUP >>> chmsee_get_charset = %s", charset);
-                if (charset && strlen(charset)) {
-                        FindCharsetData *data = g_new(FindCharsetData, sizeof(FindCharsetData));
+        const gchar *charset = chmsee_get_charset(chmsee);
+        g_debug("SETUP >>> chmsee_get_charset = %s", charset);
+        if (charset && strlen(charset) && g_strcmp0(charset, "Auto")) {
+                FindCharsetData *data = g_new(FindCharsetData, sizeof(FindCharsetData));
 
-                        data->combobox = GTK_COMBO_BOX (cmb_lang);
-                        data->charset = charset;
+                data->combobox = GTK_COMBO_BOX (cmb_lang);
+                data->charset = charset;
 
-                        gtk_tree_model_foreach(GTK_TREE_MODEL (cmb_model),
-                                               (GtkTreeModelForeachFunc) find_charset_func,
-                                               data);
-                        g_free(data);
-                } else {
-                        gtk_combo_box_set_active(GTK_COMBO_BOX (cmb_lang), 0);
-                }
-
-                gtk_widget_set_sensitive(cmb_lang, TRUE);
+                gtk_tree_model_foreach(GTK_TREE_MODEL (cmb_model),
+                                       (GtkTreeModelForeachFunc) find_charset_func,
+                                       data);
+                g_free(data);
+        } else {
+                gtk_combo_box_set_active(GTK_COMBO_BOX (cmb_lang), 0);
         }
 
         g_signal_connect(G_OBJECT (cmb_lang),
@@ -378,7 +370,9 @@ setup_window_new(Chmsee *chmsee)
                          "toggled",
                          G_CALLBACK (startup_lastfile_toggled_cb),
                          chmsee);
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (startup_lastfile_chkbtn), chmsee_get_startup_lastfile(chmsee));
+
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (startup_lastfile_chkbtn),
+                                     chmsee_get_startup_lastfile(chmsee));
 
         GtkWidget *close_button = BUILDER_WIDGET (builder, "setup_close");
         g_signal_connect(G_OBJECT (close_button),
